# -*- coding: utf-8 -*-

"""Provider for NBER

(US) National Bureau of Economic Research

Author: G.J.J. van den Burg
License: See LICENSE file
Copyright: 2020, G.J.J. van den Burg

"""

import re

from ._base import Provider
from ._info import Informer
from ..exceptions import URLResolutionError


class NBERInformer(Informer):
    def _format_year(self, soup_date):
        return soup_date.split("-")[0]


class NBER(Provider):

    re_abs = "https?://www\.nber\.org/papers/(?P<ref>[a-z0-9]+)$"
    re_pdf = "https?://www\.nber\.org/papers/(?P<ref>[a-z0-9]+)\.pdf$"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.informer = NBERInformer()

    def get_abs_pdf_urls(self, url):
        if re.match(self.re_abs, url):
            abs_url = url
            pdf_url = url + ".pdf"
        elif re.match(self.re_pdf, url):
            pdf_url = url
            abs_url = url[: -len(".pdf")]
        else:
            raise URLResolutionError("NBER", url)
        return abs_url, pdf_url

    def validate(src):
        return re.match(NBER.re_abs, src) or re.match(NBER.re_pdf, src)



# -*- coding: utf-8 -*-

"""Exceptions for paper2remarkable

"""

from . import GITHUB_URL

from subprocess import CalledProcessError

GH_MSG = "\n\nIf you think this might be a bug, please raise an issue on GitHub at: {url}".format(
    url=GITHUB_URL
)


class Error(Exception):
    """Base class for exceptions in p2r."""


class URLResolutionError(Error):
    """Exception raised when the abstract/pdf urls can't be resolved.

    Attributes
    ----------
    provider : str
        Name of the provider where the error occurred

    url : str
        Original url provided

    """

    def __init__(self, provider, url, reason=None):
        self.provider = provider
        self.url = url
        self.reason = None

    def __str__(self):
        msg = "ERROR: Couldn't figure out {provider} URLs from provided url: {url}".format(
            provider=self.provider, url=self.url
        )
        if self.reason:
            msg += "\nReason: {reason}".format(reason=self.reason)
        msg += GH_MSG
        return msg


class FilenameMissingError(Error):
    """Exception raised for providers that need a filename to be provided"""

    def __init__(self, provider, url, reason=None):
        self.provider = provider
        self.url = url
        self.reason = reason

    def __str__(self):
        msg = "ERROR: Couldn't determine a filename from {url} for provider {provider}".format(
            provider=self.provider, url=self.url
        )
        if self.reason:
            msg += "\nReason: {reason}".format(reason=self.reason)
        msg += GH_MSG
        return msg


class FileTypeError(Error):
    """Exception raised when we have a mismatch in filetype."""

    def __init__(self, filename, filetype):
        self.filename = filename
        self.filetype = filetype

    def __str__(self):
        msg = "ERROR: File {filename} isn't of type {filetype}.".format(
            filename=self.filename, filetype=self.filetype
        )
        msg += GH_MSG
        return msg


class RemarkableError(Error):
    """Exceptions raised when interacting with the reMarkable fails."""

    def __init__(self, message):
        self.message = message

    def __str__(self):
        msg = "ERROR: {message}".format(message=self.message)
        msg += GH_MSG
        return msg


class _CalledProcessError(CalledProcessError):
    """Exception raised when subprocesses fail.

    We subclass the CalledProcessError so we can add our custom error message.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def __str__(self):
        parent = super().__str__()
        msg = parent + GH_MSG
        return msg



# -*- coding: utf-8 -*-

"""Command line interface

Author: G.J.J. van den Burg
License: See LICENSE file
Copyright: 2019, G.J.J. van den Burg

"""

import argparse
import sys

from . import __version__, GITHUB_URL

from .providers import providers, LocalFile
from .utils import follow_redirects, is_url


def parse_args():
    parser = argparse.ArgumentParser(
        description="Paper2reMarkable version %s" % __version__
    )
    parser.add_argument(
        "-b",
        "--blank",
        help="Add a blank page after every page of the PDF",
        action="store_true",
    )
    parser.add_argument(
        "-c",
        "--center",
        help="Center the PDF on the page, instead of left align",
        action="store_true",
    )
    parser.add_argument(
        "-d",
        "--debug",
        help="debug mode, doesn't upload to reMarkable",
        action="store_true",
    )
    parser.add_argument(
        "-n",
        "--no-upload",
        help="don't upload to the reMarkable, save the output in current working dir",
        action="store_true",
    )
    parser.add_argument(
        "-p",
        "--remarkable-path",
        help="directory on reMarkable to put the file (created if missing, default: /)",
        dest="remarkable_dir",
        default="/",
    )
    parser.add_argument(
        "-r",
        "--right",
        help="Right align so the menu doesn't cover it",
        action="store_true",
    )
    parser.add_argument(
            '-k',
            '--no-crop',
            help="Don't crop the pdf file",
            action="store_true"
            )

    parser.add_argument(
        "-v", "--verbose", help="be verbose", action="store_true"
    )
    parser.add_argument(
        "-V",
        "--version",
        help="Show version and exit",
        action="version",
        version=__version__,
    )
    parser.add_argument(
        "--filename",
        help="Filename to use for the file on reMarkable",
        default=None,
    )
    parser.add_argument(
        "--gs", help="path to gs executable (default: gs)", default="gs"
    )
    parser.add_argument(
        "--pdftoppm",
        help="path to pdftoppm executable (default: pdftoppm)",
        default="pdftoppm",
    )
    parser.add_argument(
        "--pdftk",
        help="path to pdftk executable (default: pdftk)",
        default="pdftk",
    )
    parser.add_argument(
        "--rmapi",
        help="path to rmapi executable (default: rmapi)",
        default="rmapi",
    )
    parser.add_argument(
        "input", help="URL to a paper or the path of a local PDF file"
    )
    return parser.parse_args()


def exception(msg):
    print("ERROR: " + msg, file=sys.stderr)
    print("Error occurred. Exiting.", file=sys.stderr)
    print("", file=sys.stderr)
    print(
        "If you think this might be a bug, please raise an issue on GitHub: %s"
        % GITHUB_URL,
        file=sys.stderr,
    )
    print("", file=sys.stderr)
    raise SystemExit(1)


def main():
    args = parse_args()
    cookiejar = None

    if args.center and args.right:
        exception("Can't center and right align at the same time!")

    if args.center and args.no_crop:
        exception("Can't center and not crop at the same time!")

    if args.right and args.no_crop:
        exception("Can't right align and not crop at the same time!")

    if LocalFile.validate(args.input):
        # input is a local file
        provider = LocalFile
    elif is_url(args.input):
        # input is a url
        url, cookiejar = follow_redirects(args.input)
        provider = next((p for p in providers if p.validate(url)), None)
    else:
        # not a proper URL or non-existent file
        exception(
            "Couldn't figure out what source you mean. If it's a "
            "local file, make sure it exists."
        )

    if provider is None:
        exception("Input not valid, no provider can handle this source.")

    prov = provider(
        verbose=args.verbose,
        upload=not args.no_upload,
        debug=args.debug,
        center=args.center,
        right=args.right,
        blank=args.blank,
        no_crop=args.no_crop,
        remarkable_dir=args.remarkable_dir,
        rmapi_path=args.rmapi,
        pdftoppm_path=args.pdftoppm,
        pdftk_path=args.pdftk,
        gs_path=args.gs,
        cookiejar=cookiejar,
    )

    prov.run(args.input, filename=args.filename)
